#!/usr/bin/perl -w
#

# Parse JSON stats from BTC Guild for Cacti
#
# Can be invoked in three seperate modes for each of the stats: User, Miners and Pools
#
# Sample invocation:
#
# User
#   query_btcguild_stats.pl HOST API_KEY user 
#
# Miners
#   query_btcguild_stats.pl HOST API_KEY miners index
#
# Pools
#   query_btcguild_stats.pl HOST API_KEY pool index
#
#
# v0.01	6/24/2011 jintuNO@SPAMpraecogito.com
#	First working version. Only supports btcguild stats for now. No JSON query caching support (Slow!). 
 

use strict;
use JSON -support_by_pp;
use HTTP::Request;
use LWP::UserAgent;

my $host = $ARGV[0];
my $api_key = $ARGV[1];
my $mode = $ARGV[2];

unless ((defined $host) && (defined $api_key) && (($ARGV[2] eq "user") || ($ARGV[2] eq "workers") || ($ARGV[2] eq "pool") || ($ARGV[2] eq "total_all_pools"))) {
  # Missing input params
  print "usage:\n\n./query_btcguild_stats.pl HOST API_KEY user\n\n" .
        "./query_btcguild_stats.pl HOST API_KEY workers index\n" . 
        "./query_btcguild_stats.pl HOST API_KEY workers num_indexes\n" . 
        "./query_btcguild_stats.pl HOST API_KEY workers query {worker_name,hash_rate,round_shares,round_stales,reset_shares,reset_stales,total_shares,total_stales,last_share,blocks_found}\n" . 
        "./query_btcguild_stats.pl HOST API_KEY workers get {worker_name,hash_rate,round_shares,round_stales,reset_shares,reset_stales,total_shares,total_stales,last_share,blocks_found} WORKER\n\n" . 
        "./query_btcguild_stats.pl HOST API_KEY pool index\n" .
        "./query_btcguild_stats.pl HOST API_KEY pool num_indexes\n" .
        "./query_btcguild_stats.pl HOST API_KEY pool query {pool_name,speed}\n" .
        "./query_btcguild_stats.pl HOST API_KEY pool get {pool_name,speed} POOL\n\n" .
        "./query_btcguild_stats.pl HOST API_KEY total_all_pools\n";
  exit;
}

my $json_url = "https://$host/api.php?api_key=$api_key";

eval{
    # download the json page:
    #print "Getting json $json_url\n";
    my $request = HTTP::Request->new(GET => $json_url);
    my $ua = LWP::UserAgent->new;
    my $response = $ua->request($request);
    my $content = $response->content();
    my $json = new JSON;
 
    # these are some nice json options to relax restrictions a bit:
    my $json_text = $json->allow_nonref->utf8->relaxed->escape_slash->loose->allow_singlequote->allow_barekey->decode($content);
     
    # Cacti needs whole numbers, so this needs to be divided by 100000000 to get the original value
    if ($mode eq "user") {
       print "confirmed_rewards:" . $json_text->{user}->{confirmed_rewards} * 100000000 . " " .
        "unconfirmed_rewards:" . $json_text->{user}->{unconfirmed_rewards} * 100000000 . " " .
        "estimated_rewards:" . $json_text->{user}->{estimated_rewards} * 100000000 . " " .
        "payouts:" . $json_text->{user}->{payouts} * 100000000;
    }
    elsif($mode eq "workers") {
      # Enumerate the worker names so we don't have indexing issues
      my %workerid;
      foreach my $worker(%{$json_text->{workers}}) {
        if (defined $json_text->{workers}->{$worker}->{worker_name}) {
          $workerid{$json_text->{workers}->{$worker}->{worker_name}} = $worker;
        }
      }     
      if (($ARGV[3] eq "get") && (defined $ARGV[4]) && (defined $ARGV[5])) {
        if (defined $json_text->{workers}->{$workerid{$ARGV[5]}}->{$ARGV[4]}) {
          if ($ARGV[4] eq "hash_rate") {
            # need to make this a whole number for Cacti
            printf ("%d", $json_text->{workers}->{$workerid{$ARGV[5]}}->{$ARGV[4]});  
          }
          elsif ($ARGV[4] eq "last_share") {
            print convert_time($json_text->{workers}->{$workerid{$ARGV[5]}}->{$ARGV[4]});
          }
          else {
            print $json_text->{workers}->{$workerid{$ARGV[5]}}->{$ARGV[4]};
          }
        } 
      }
      elsif (($ARGV[3] eq "query") && (defined $ARGV[4])) {
        foreach my $worker_name (keys %workerid) {
          if (($ARGV[4] eq "worker_name") && (defined $json_text->{workers}->{$workerid{"$worker_name"}}->{worker_name})) {
      	    print "$worker_name:$json_text->{workers}->{$workerid{$worker_name}}->{worker_name}\n";
          }
          elsif (($ARGV[4] eq "hash_rate") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{hash_rate})) {
      	    print "$worker_name:" . sprintf("%d",$json_text->{workers}->{$workerid{$worker_name}}->{hash_rate}) ."\n";
          }
          elsif (($ARGV[4] eq "round_shares") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{round_shares})) {
      	    print "$worker_name:$json_text->{workers}->{$workerid{$worker_name}}->{round_shares}\n";
          }
          elsif (($ARGV[4] eq "round_stales") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{round_stales})) {
      	    print "$worker_name:$json_text->{workers}->{$workerid{$worker_name}}->{round_stales}\n";
          }
          elsif (($ARGV[4] eq "reset_shares") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{reset_shares})) {
      	    print "$worker_name:$json_text->{workers}->{$workerid{$worker_name}}->{reset_shares}\n";
          }
          elsif (($ARGV[4] eq "reset_stales") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{reset_stales})) {
      	    print "$worker_name:$json_text->{workers}->{$workerid{$worker_name}}->{reset_stales}\n";
          }
          elsif (($ARGV[4] eq "total_shares") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{total_shares})) {
      	    print "$worker_name:$json_text->{workers}->{$workerid{$worker_name}}->{total_shares}\n";
          }
          elsif (($ARGV[4] eq "total_stales") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{total_stales})) {
      	    print "$worker_name:$json_text->{workers}->{$workerid{$worker_name}}->{total_stales}\n";
          }
          elsif (($ARGV[4] eq "last_share") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{last_share})) {
      	    print "$worker_name:" . convert_time($json_text->{workers}->{$workerid{$worker_name}}->{last_share}) ."\n";
          }
          elsif (($ARGV[4] eq "blocks_found") && (defined $json_text->{workers}->{$workerid{$worker_name}}->{blocks_found})) {
      	    print "$worker_name:$json_text->{workers}->{$workerid{$worker_name}}->{blocks_found}\n";
          }
        }
      }
      elsif ($ARGV[3] eq "index") {
        foreach my $worker_name (keys %workerid) {
          print "$worker_name\n";
        }
      }
      elsif ($ARGV[3] eq "num_indexes") {
        print scalar(keys %workerid);
      }
    }
    elsif ($mode eq "pool") {
      if (($ARGV[3] eq "get") && (defined $json_text->{pool}->{$ARGV[5] . "_speed"})) {
         if ($ARGV[4] eq "speed") {
            printf ("%d", $json_text->{pool}->{$ARGV[5] . "_speed"});
         }
         elsif ($ARGV[4] eq "pool_name") {
            print $ARGV[5];
         }
      }
      else {
        my $num_pools = 0;
        foreach my $pool (%{$json_text->{pool}}) {
           if (my ($pname) = $pool =~ m/(.*)_speed/) {
              if ($ARGV[3] eq "index") {
                 print "$pname\n";
              }
              elsif ($ARGV[3] eq "query") {
                 if ($ARGV[4] eq "speed") {
                    print "$pname:" . sprintf ("%d", $json_text->{pool}->{$pname . "_speed"}) . "\n"; 
                 }
                 elsif ($ARGV[4] eq "pool_name") {
                    print "$pname:$pname\n";
                 }
              }
              $num_pools++;
           }
        }
        if ($ARGV[3] eq "num_indexes") {
           print $num_pools;
        }
      }
    }
    elsif ($mode eq "total_all_pools") {
      print "hash_rate:" . sprintf ("%d", $json_text->{pool}->{hash_rate}) . " " .
         "active_workers:$json_text->{pool}->{active_workers} " .
         "round_shares:$json_text->{pool}->{round_shares} " .
         "round_time:" . convert_time($json_text->{pool}->{round_time});
    }
  };
  # catch crashes:
  if($@){
    print "Crash detected: $@\n";
  }

# Convert HH:MM:SS into seconds
sub convert_time {
   my ($time_in) = @_;
   my ($hours,$minutes,$seconds) = $time_in =~ m/(\d*):(\d*):(\d*)/;
   return $seconds + ($minutes * 60) + ($hours * 60 * 60); 
} 
